@@ -12,6 +12,7 @@
 //---------------------------------------------------------------------------
 #include "MediaInfo/File__Base.h"
 #include "MediaInfo/File__Analyse_Automatic.h"
+#include "MediaInfo/File__Analyze_Element.h"
 #include "ZenLib/BitStream_Fast.h"
 #include "ZenLib/BitStream_LE.h"
 #if MEDIAINFO_IBIUSAGE
@@ -22,6 +23,7 @@
     #include <aescpp.h>
 #endif //MEDIAINFO_AES
 #include "MediaInfo/HashWrapper.h"
+#include <cstring>
 //---------------------------------------------------------------------------
 
 namespace MediaInfoLib
@@ -55,8 +57,7 @@ public :
     void    Open_Buffer_Init        (File__Analyze* Sub);
     void    Open_Buffer_Init        (File__Analyze* Sub, int64u File_Size);
     void    Open_Buffer_OutOfBand   (                    const int8u* Buffer, size_t Buffer_Size) {File__Analyze::Buffer=Buffer; File__Analyze::Buffer_Size=Buffer_Size; Element_Offset=0; Element_Size=Buffer_Size; Read_Buffer_OutOfBand(); File__Analyze::Buffer=NULL; File__Analyze::Buffer_Size=0; Element_Offset=0; Element_Size=0;}
-    void    Open_Buffer_OutOfBand   (File__Analyze* Sub, const int8u* Buffer, size_t Buffer_Size);
-    void    Open_Buffer_OutOfBand   (File__Analyze* Sub) {if (Element_Offset<=Element_Size) Open_Buffer_OutOfBand(Sub, Buffer+Buffer_Offset+(size_t)Element_Offset, (size_t)(Element_Size-Element_Offset)); Element_Offset=Element_Size;}
+    void    Open_Buffer_OutOfBand   (File__Analyze* Sub);
     void    Open_Buffer_Continue    (                    const int8u* Buffer, size_t Buffer_Size);
     void    Open_Buffer_Continue    (File__Analyze* Sub, const int8u* Buffer, size_t Buffer_Size, bool IsNewPacket=true, float64 Ratio=1.0);
     void    Open_Buffer_Continue    (File__Analyze* Sub, size_t Buffer_Size) {if (Element_Offset+Buffer_Size<=Element_Size) Open_Buffer_Continue(Sub, Buffer+Buffer_Offset+(size_t)Element_Offset, Buffer_Size); Element_Offset+=Buffer_Size;}
@@ -79,7 +80,7 @@ public :
     //***************************************************************************
 
     //In
-    Ztring ParserName;
+    std::string ParserName;
     #if MEDIAINFO_EVENTS
         size_t  StreamIDs_Size;
         int64u  StreamIDs[16];
@@ -232,7 +233,7 @@ protected :
 
     //Buffer
     virtual void Read_Buffer_Init ()          {}; //Temp, should be in File__Base caller
-    virtual void Read_Buffer_OutOfBand ()     {}; //Temp, should be in File__Base caller
+    virtual void Read_Buffer_OutOfBand ()     {Open_Buffer_Continue(Buffer, Buffer_Size);} //Temp, should be in File__Base caller
     virtual void Read_Buffer_Continue ()      {}; //Temp, should be in File__Base caller
     virtual void Read_Buffer_AfterParsing ()  {}; //Temp, should be in File__Base caller
     #if MEDIAINFO_SEEK
@@ -335,29 +336,46 @@ protected :
     inline void Element_Name (const char*   Name) {Element_Name(Ztring().From_UTF8(Name));}
 
     //Elements - Info
-    void Element_Info (const Ztring &Parameter);
-    inline void Element_Info (const char*   Parameter) {if (Config_Trace_Level<1) return; Element_Info(Ztring().From_UTF8(Parameter));}
-    inline void Element_Info (const char*   Parameter, const char*   Measure)      {if (Config_Trace_Level<1) return; Element_Info(Ztring().From_UTF8(Parameter)+Ztring().From_UTF8(Measure));}
-    inline void Element_Info (int8s         Parameter, const char*   Measure=NULL) {if (Config_Trace_Level<1) return; Element_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
-    inline void Element_Info (int8u         Parameter, const char*   Measure=NULL) {if (Config_Trace_Level<1) return; Element_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
-    inline void Element_Info (int16s        Parameter, const char*   Measure=NULL) {if (Config_Trace_Level<1) return; Element_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
-    inline void Element_Info (int16u        Parameter, const char*   Measure=NULL) {if (Config_Trace_Level<1) return; Element_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
-    inline void Element_Info (int32s        Parameter, const char*   Measure=NULL) {if (Config_Trace_Level<1) return; Element_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
-    inline void Element_Info (int32u        Parameter, const char*   Measure=NULL) {if (Config_Trace_Level<1) return; Element_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
-    inline void Element_Info (int64s        Parameter, const char*   Measure=NULL) {if (Config_Trace_Level<1) return; Element_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
-    inline void Element_Info (int64u        Parameter, const char*   Measure=NULL) {if (Config_Trace_Level<1) return; Element_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
-    inline void Element_Info (int128u       Parameter, const char*   Measure=NULL) {if (Config_Trace_Level<1) return; Element_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
+#if MEDIAINFO_TRACE
+    template<typename T>
+    void Element_Info (T Parameter, const char* Measure=NULL, int8u AfterComma=3)
+    {
+        if (Config_Trace_Level<1)
+            return;
+
+        //Needed?
+        if (Config_Trace_Level<=0.7)
+            return;
+
+        Element[Element_Level].TraceNode.Infos.push_back(new element_details::Element_Node_Info(Parameter, Measure, AfterComma));
+    }
+
+    void Element_Info (const char* Parameter, const char* Measure=NULL, int8u AfterComma=3)
+    {
+        if (Config_Trace_Level<1)
+            return;
+
+        //Needed?
+        if (Config_Trace_Level<=0.7)
+            return;
+
+        if (Parameter && std::string(Parameter) == "NOK")
+            Element[Element_Level].TraceNode.HasError = true;
+
+        Element[Element_Level].TraceNode.Infos.push_back(new element_details::Element_Node_Info(Parameter, Measure, AfterComma));
+    }
+#endif //MEDIAINFO_TRACE
+
     #ifdef SIZE_T_IS_LONG
-    inline void Element_Info (size_t        Parameter, const char*   Measure=NULL) {Element_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
+    inline void Element_Info (size_t        Parameter, const char*   Measure=NULL) {Element_Info(Ztring::ToZtring(Parameter), Measure);}
     #endif //SIZE_T_IS_LONG
-    inline void Element_Info (float32       Parameter, int8u AfterComma=3, const char*   Measure=NULL) {if (Config_Trace_Level<1) return; Element_Info(Ztring::ToZtring(Parameter, AfterComma)+Ztring().From_UTF8(Measure));}
-    inline void Element_Info (float64       Parameter, int8u AfterComma=3, const char*   Measure=NULL) {if (Config_Trace_Level<1) return; Element_Info(Ztring::ToZtring(Parameter, AfterComma)+Ztring().From_UTF8(Measure));}
     #define Element_Info1(_A) Element_Info(_A)
     #define Element_Info2(_A,_B) Element_Info(_A, _B)
     #define Element_Info3(_A,_B,_C) Element_Info(_A, _B, _C)
     #define Element_Info1C(_CONDITION,_A) if (_CONDITION) Element_Info(_A)
     inline void Element_Info_From_Milliseconds (int64u Parameter)                  {if (Config_Trace_Level<1) return; Element_Info(Ztring().Duration_From_Milliseconds(Parameter));}
-    void Element_Parser (const Ztring &Name);
+
+    void Element_Parser (const char* Name);
 
     //Elements - End
     inline void Element_End () {Element_End_Common_Flush();}
@@ -375,7 +393,6 @@ protected :
     //Element - Common
     void   Element_End_Common_Flush();
     void   Element_End_Common_Flush_Details();
-    Ztring Element_End_Common_Flush_Build();
 public :
 
     //***************************************************************************
@@ -394,50 +411,91 @@ public :
         Ztring::ToZtring(Value).MakeUpperCase()+__T(" (0x")+Ztring::ToZtring(Value, 16).MakeUpperCase()+__T(")")
 
     //Param - Main
-    void Param      (const Ztring &Parameter, const Ztring& Value);
-    inline void Param      (const char*   Parameter, const Ztring& Value) {if (Trace_Activated) Param(Ztring().From_Local(Parameter), Value);};
-    inline void Param      (const char*   Parameter, const std::string& Value) { if (Trace_Activated) Param(Parameter, Ztring().From_Local(Value.c_str()));}
-    inline void Param      (const char*   Parameter, const char*   Value, size_t Value_Size=Unlimited, bool Utf8=true) {if (Trace_Activated) Param(Parameter, ToZtring(Value, Value_Size, Utf8));}
-    inline void Param      (const char*   Parameter, const int8u*  Value, size_t Value_Size=Unlimited, bool Utf8=true) {if (Trace_Activated) Param(Parameter, (const char*)Value, Value_Size, Utf8);}
-    inline void Param      (const char*   Parameter, bool   Value) {if (Trace_Activated) {if (Value) Param(Parameter, "Yes"); else Param(Parameter, "No");}}
-    inline void Param      (const char*   Parameter, int8u  Value) {if (Trace_Activated) Param(Parameter, Ztring::ToZtring(Value)+__T(" (0x")+Ztring().From_CC1(Value)+__T(")"));}
-    inline void Param      (const char*   Parameter, int8s  Value) {if (Trace_Activated) Param(Parameter, Ztring::ToZtring(Value)+__T(" (0x")+Ztring().From_CC1(Value)+__T(")"));}
-    inline void Param      (const char*   Parameter, int16u Value) {if (Trace_Activated) Param(Parameter, Ztring::ToZtring(Value)+__T(" (0x")+Ztring().From_CC2(Value)+__T(")"));}
-    inline void Param      (const char*   Parameter, int16s Value) {if (Trace_Activated) Param(Parameter, Ztring::ToZtring(Value)+__T(" (0x")+Ztring().From_CC2(Value)+__T(")"));}
-    inline void Param      (const char*   Parameter, int32u Value) {if (Trace_Activated) Param(Parameter, Ztring::ToZtring(Value)+__T(" (0x")+Ztring::ToZtring(Value, 16)+__T(")"));}
-    inline void Param      (const char*   Parameter, int32s Value) {if (Trace_Activated) Param(Parameter, Ztring::ToZtring(Value)+__T(" (0x")+Ztring::ToZtring(Value, 16)+__T(")"));}
-    inline void Param      (const char*   Parameter, int64u Value) {if (Trace_Activated) Param(Parameter, Ztring::ToZtring(Value)+__T(" (0x")+Ztring::ToZtring(Value, 16)+__T(")"));}
-    inline void Param      (const char*   Parameter, int64s Value) {if (Trace_Activated) Param(Parameter, Ztring::ToZtring(Value)+__T(" (0x")+Ztring::ToZtring(Value, 16)+__T(")"));}
-    inline void Param      (const char*   Parameter, int128u Value){if (Trace_Activated) Param(Parameter, Ztring::ToZtring(Value)+__T(" (0x")+Ztring::ToZtring(Value, 16)+__T(")"));}
-    inline void Param_GUID (const char*   Parameter, int128u Value){if (Trace_Activated) Param(Parameter, Ztring().From_GUID(Value));}
-    inline void Param_UUID (const char*   Parameter, int128u Value){if (Trace_Activated) Param(Parameter, Ztring().From_UUID(Value));}
-    #ifdef SIZE_T_IS_LONG
-    inline void Param      (const char*   Parameter, size_t Value, intu Radix=16) {if (Trace_Activated) Param(Parameter, Ztring::ToZtring(Value, Radix).MakeUpperCase()+__T(" (")+Ztring::ToZtring(Value, 10).MakeUpperCase()+__T(")"));}
-    #endif //SIZE_T_IS_LONG
-    inline void Param      (const char*   Parameter, float32 Value, int8u AfterComma=3) {if (Trace_Activated) Param(Parameter, Ztring::ToZtring(Value, AfterComma));}
-    inline void Param      (const char*   Parameter, float64 Value, int8u AfterComma=3) {if (Trace_Activated) Param(Parameter, Ztring::ToZtring(Value, AfterComma));}
-    inline void Param      (const char*   Parameter, float80 Value, int8u AfterComma=3) {if (Trace_Activated) Param(Parameter, Ztring::ToZtring(Value, AfterComma));}
-    inline void Param      (const int32u  Parameter, const Ztring& Value) {if (Trace_Activated) Param(Ztring().From_CC4(Parameter), Value);};
-    inline void Param      (const int16u  Parameter, const Ztring& Value) {if (Trace_Activated) Param(Ztring().From_CC2(Parameter), Value);};
+    template<typename T>
+    inline void Param(const std::string &Parameter, T Value, int8u GenericOption=(int8u)-1)
+    {
+        if (!Trace_Activated)
+            return;
+
+        if (Config_Trace_Level==0 || !(Trace_Layers.to_ulong()&Config_Trace_Layers.to_ulong()))
+            return ;
+
+        //Coherancy
+        if (Element[Element_Level].UnTrusted)
+            return ;
+
+        element_details::Element_Node *node = new element_details::Element_Node;
+        node->Set_Name(Parameter);
+        node->Pos = File_Offset+Buffer_Offset+Element_Offset;
+        if (BS_Size)
+        {
+            int64u BS_BitOffset = BS_Size-BS->Remain();
+            if (GenericOption != (int8u)-1)
+                BS_BitOffset -= GenericOption;
+            node->Pos += BS_BitOffset>>3; //Including Bits to Bytes
+        }
+        node->Value.set_Option(GenericOption);
+        node->Value = Value;
+        Element[Element_Level].TraceNode.Current_Child = Element[Element_Level].TraceNode.Children.size();
+        Element[Element_Level].TraceNode.Children.push_back(node);
+    }
+
+    inline void Param      (const char*   Parameter, const char*   Value, size_t Value_Size, bool Utf8=true) {Param(Parameter, ToZtring(Value, Value_Size, Utf8));}
+    inline void Param      (const char*   Parameter, const int8u*  Value, size_t Value_Size, bool Utf8=true) {Param(Parameter, (const char*)Value, Value_Size, Utf8);}
+    inline void Param_GUID (const char*   Parameter, int128u Value){Param(Parameter, Ztring().From_GUID(Value));}
+    inline void Param_UUID (const char*   Parameter, int128u Value){Param(Parameter, Ztring().From_UUID(Value));}
+    /* #ifdef SIZE_T_IS_LONG */
+    /* inline void Param      (const char*   Parameter, size_t Value, intu Radix) {if (Trace_Activated) Param(Parameter, Ztring::ToZtring(Value, Radix).MakeUpperCase()+__T(" (")+Ztring::ToZtring(Value, 10).MakeUpperCase()+__T(")"));} */
+    /* #endif //SIZE_T_IS_LONG */
+    inline void Param      (const int32u  Parameter, const Ztring& Value) {if (Trace_Activated) Param(Ztring().From_CC4(Parameter).To_UTF8(), Value.To_UTF8());};
+    inline void Param      (const int16u  Parameter, const Ztring& Value) {if (Trace_Activated) Param(Ztring().From_CC2(Parameter).To_UTF8(), Value.To_UTF8());};
     #define Param1(_A) Param_(_A)
     #define Param2(_A,_B) Param(_A, _B)
     #define Param3(_A,_B,_C) Param(_A, _B, _C)
 
     //Param - Info
-    void Param_Info (const Ztring &Parameter);
-    inline void Param_Info (const char*   Parameter) {if (Trace_Activated) Param_Info(Ztring().From_UTF8(Parameter));}
-    inline void Param_Info (const char*   Parameter, const char*   Measure)      {if (Trace_Activated) Param_Info(Ztring().From_UTF8(Parameter)+Ztring().From_UTF8(Measure));}
-    inline void Param_Info (int64u        Parameter, const char*   Measure=NULL) {if (Trace_Activated) Param_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
-    inline void Param_Info (int64s        Parameter, const char*   Measure=NULL) {if (Trace_Activated) Param_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
-    inline void Param_Info (int32u        Parameter, const char*   Measure=NULL) {if (Trace_Activated) Param_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
-    inline void Param_Info (int32s        Parameter, const char*   Measure=NULL) {if (Trace_Activated) Param_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
-    inline void Param_Info (int16u        Parameter, const char*   Measure=NULL) {if (Trace_Activated) Param_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
-    inline void Param_Info (int16s        Parameter, const char*   Measure=NULL) {if (Trace_Activated) Param_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
-    inline void Param_Info (int8u         Parameter, const char*   Measure=NULL) {if (Trace_Activated) Param_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
-    inline void Param_Info (int8s         Parameter, const char*   Measure=NULL) {if (Trace_Activated) Param_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
-    inline void Param_Info (float32       Parameter, int8u AfterComma=3, const char*   Measure=NULL) {if (Trace_Activated) Param_Info(Ztring::ToZtring(Parameter, AfterComma)+Ztring().From_UTF8(Measure));}
-    inline void Param_Info (float64       Parameter, int8u AfterComma=3, const char*   Measure=NULL) {if (Trace_Activated) Param_Info(Ztring::ToZtring(Parameter, AfterComma)+Ztring().From_UTF8(Measure));}
-    inline void Param_Info (float80       Parameter, int8u AfterComma=3, const char*   Measure=NULL) {if (Trace_Activated) Param_Info(Ztring::ToZtring(Parameter, AfterComma)+Ztring().From_UTF8(Measure));}
+#if MEDIAINFO_TRACE
+    template<typename T>
+    void Param_Info(T Parameter, const char* Measure=NULL, int8u AfterComma=3)
+    {
+        //Coherancy
+        if (!Trace_Activated)
+            return;
+        if (Element[Element_Level].UnTrusted)
+            return;
+        if (Config_Trace_Level<=0.7)
+            return;
+
+        // if (!(Trace_Layers.to_ulong()&Config_Trace_Layers.to_ulong()) || Element[Element_Level].TraceNode.Details.size()>64*1024*1024)
+        //     return;
+        int32s child = Element[Element_Level].TraceNode.Current_Child;
+        if (child >= 0 && Element[Element_Level].TraceNode.Children[child])
+            Element[Element_Level].TraceNode.Children[child]->Infos.push_back(new element_details::Element_Node_Info(Parameter, Measure, AfterComma));
+        else
+            Element[Element_Level].TraceNode.Infos.push_back(new element_details::Element_Node_Info(Parameter, Measure, AfterComma));
+    }
+
+    void Param_Info(const char* Parameter, const char* Measure=NULL, int8u AfterComma=3)
+    {
+        //Coherancy
+        if (!Trace_Activated)
+            return;
+        if (Element[Element_Level].UnTrusted)
+            return;
+        if (Config_Trace_Level<=0.7)
+            return;
+
+        if (Parameter && std::string(Parameter) == "NOK")
+            Element[Element_Level].TraceNode.HasError = true;
+
+        int32s child = Element[Element_Level].TraceNode.Current_Child;
+        if (child >= 0 && Element[Element_Level].TraceNode.Children[child])
+            Element[Element_Level].TraceNode.Children[child]->Infos.push_back(new element_details::Element_Node_Info(Parameter, Measure, AfterComma));
+        else
+            Element[Element_Level].TraceNode.Infos.push_back(new element_details::Element_Node_Info(Parameter, Measure, AfterComma));
+    }
+#endif //MEDIAINFO_TRACE
+
     #ifdef SIZE_T_IS_LONG
     inline void Param_Info (size_t        Parameter, const char*   Measure=NULL) {if (Trace_Activated) Param_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
     #endif //SIZE_T_IS_LONG
@@ -449,11 +507,16 @@ public :
     #define Param_Info3C(_CONDITION,_A,_B,_C) if (_CONDITION) Param_Info(_A, _B, _C)
     inline void Param_Info_From_Milliseconds (int64u Parameter)                  {if (Trace_Activated) Param_Info(Ztring().Duration_From_Milliseconds(Parameter));}
 
+    //***************************************************************************
+    // Element Node
+    //***************************************************************************
+    element_details::Element_Node *Get_Trace_Node(size_t level = 0);
+
     //***************************************************************************
     // Information
     //***************************************************************************
 
-    void Info (const Ztring& Value, size_t Element_Level_Minus=0);
+    void Info (const std::string& Value, size_t Element_Level_Minus=0);
 
     //***************************************************************************
     // Big Endian (Integer, Float, Fixed-Point)
@@ -1026,7 +1089,13 @@ public :
     #ifdef SIZE_T_IS_LONG
     inline void Fill (stream_t StreamKind, size_t StreamPos, const char* Parameter, size_t         Value, int8u Radix=10, bool Replace=false) {Fill(StreamKind, StreamPos, Parameter, Ztring::ToZtring(Value, Radix).MakeUpperCase(), Replace);}
     #endif //SIZE_T_IS_LONG
-    ZtringListList Fill_Temp;
+    struct fill_temp_item
+    {
+        Ztring Parameter;
+        Ztring Value;
+    };
+    void Fill_SetOptions(stream_t StreamKind, size_t StreamPos, const char* Parameter, const char* Options);
+    vector<fill_temp_item> Fill_Temp[Stream_Max+1]; // +1 because Fill_Temp[Stream_Max] is used when StreamKind is unknown
     void Fill_Flush ();
     static size_t Fill_Parameter(stream_t StreamKind, generic StreamPos);
 
@@ -1068,8 +1137,13 @@ public :
     void Element_DoNotTrust (const char* Reason);
     void Element_DoNotShow ();
     void Element_Show ();
+    void Element_Set_Remove_Children_IfNoErrors ();
+    void Element_Remove_Children_IfNoErrors ();
+    void Element_Children_IfNoErrors ();
+    void Element_DoNotShow_Children ();
+    void Element_Show_Children ();
     bool Element_Show_Get ();
-    void Element_Show_Add (const Ztring &ToShow);
+    void Element_Show_Add (File__Analyze* node);
 
     //Status
     bool Element_IsOK ();
@@ -1090,6 +1164,10 @@ public :
     void CodecID_Fill           (const Ztring &Value, stream_t StreamKind, size_t StreamPos, infocodecid_format_t Format, stream_t StreamKind_CodecID=Stream_Max);
     void PixelAspectRatio_Fill  (const Ztring &Value, stream_t StreamKind, size_t StreamPos, size_t Parameter_Width, size_t Parameter_Height, size_t Parameter_PixelAspectRatio, size_t Parameter_DisplayAspectRatio);
     void DisplayAspectRatio_Fill(const Ztring &Value, stream_t StreamKind, size_t StreamPos, size_t Parameter_Width, size_t Parameter_Height, size_t Parameter_PixelAspectRatio, size_t Parameter_DisplayAspectRatio);
+    #if MEDIAINFO_EVENTS
+    static stream_t Streamkind_Get(int8u* ParserIDs, size_t StreamIDs_Size) {if ((ParserIDs[StreamIDs_Size-1]&0xF0)==0x80) return Stream_Video; if ((ParserIDs[StreamIDs_Size-1]&0xF0)==0xA0) return Stream_Audio; return Stream_Max;}
+    stream_t Streamkind_Get() {return Streamkind_Get(ParserIDs, StreamIDs_Size);}
+    #endif //MEDIAINFO_EVENTS
 
     //***************************************************************************
     // Finalize
@@ -1204,6 +1282,9 @@ protected :
     bool Synchronize_0x000001();
 public:
     void TestContinuousFileNames(size_t CountOfFiles=24, Ztring FileExtension=Ztring(), bool SkipComputeDelay=false);
+    #if MEDIAINFO_FIXITY
+    bool FixFile(int64u FileOffsetForWriting, const int8u* ToWrite, const size_t ToWrite_Size);
+    #endif// MEDIAINFO_FIXITY
 
 private :
 
@@ -1218,9 +1299,10 @@ private :
 public: //TO CHANGE
     BitStream_Fast* BS;             //For conversion from bytes to bitstream
     BitStream*      BT;             //For conversion from bytes to bitstream
+    int64u          BS_Size;
 public : //TO CHANGE
     int64u Header_Size;             //Size of the header of the current element
-    Ztring &Details_Get(size_t Level=0) {return Element[Level].ToShow.Details;} //Direct access to details
+    Ztring Details_Get(size_t Level=0) { std::string str; if (Element[Level].TraceNode.Print(Config_Trace_Format, str, Config_LineSeparator.To_UTF8(), File_Size) < 0) return Ztring(); return Ztring().From_UTF8(str);}
     void   Details_Clear();
 protected :
     bool Trace_DoNotSave;
@@ -1228,31 +1310,11 @@ protected :
     std::bitset<32> Trace_Layers;
     void Trace_Layers_Update (size_t Layer=(size_t)-1);
 private :
-
+#if MEDIAINFO_TRACE
+    void Trace_Details_Handling(File__Analyze* Sub);
+#endif // MEDIAINFO_TRACE
     //Elements
     size_t Element_Level_Base;      //From other parsers
-
-    struct element_details
-    {
-        struct to_show
-        {
-            int64u Pos;             //Position of the element in the file
-            int64u Size;            //Size of the element (including header and sub-elements)
-            int64u Header_Size;     //Size of the header of the element
-            Ztring Name;            //Name planned for this element
-            Ztring Info;            //More info about the element
-            Ztring Details;         //The main text
-            Ztring Value;           //The value (currently used only with Trace XML)
-            bool   NoShow;          //Don't show this element
-        };
-
-        int64u  Code;               //Code filled in the file
-        int64u  Next;               //
-        bool    WaitForMoreData;    //This element is not complete, we need more data
-        bool    UnTrusted;          //This element has a problem
-        bool    IsComplete;         //This element is fully buffered, no need of more
-        to_show ToShow;
-    };
     std::vector<element_details> Element;
 
     //NextCode
@@ -1361,6 +1423,7 @@ public :
     //Hash
     #if MEDIAINFO_HASH
         HashWrapper*        Hash;
+        int64u              Hash_Offset;
         int64u              Hash_ParseUpTo;
     #endif //MEDIAINFO_HASH
 
@@ -1439,7 +1502,11 @@ public :
         {
 
     #define FILLING_BEGIN_PRECISE() \
-        if (Element_IsOK() && Element_Offset==Element_Size) \
+        if (Element_Offset!=Element_Size) \
+        { \
+            Trusted_IsNot("Size is wrong"); \
+        } \
+        if (Element_IsOK()) \
         {
 
     //Else
