@@ -113,6 +113,12 @@
     #if !defined (MEDIAINFO_MD5_NO) && !defined (MEDIAINFO_MD5_YES)
         #define MEDIAINFO_MD5_NO
     #endif
+    #if !defined (MEDIAINFO_SHA1_NO) && !defined (MEDIAINFO_SHA1_YES)
+        #define MEDIAINFO_SHA1_NO
+    #endif
+    #if !defined (MEDIAINFO_SHA2_NO) && !defined (MEDIAINFO_SHA2_YES)
+        #define MEDIAINFO_SHA2_NO
+    #endif
     #if !defined (MEDIAINFO_AES_NO) && !defined (MEDIAINFO_AES_YES)
         #define MEDIAINFO_AES_NO
     #endif
@@ -133,6 +139,19 @@
         #define MEDIAINFO_TRACE 1
     #endif
 #endif
+#if !defined(MEDIAINFO_TRACE_FFV1CONTENT)
+    #if defined(MEDIAINFO_TRACE_FFV1CONTENT_NO) && defined(MEDIAINFO_TRACE_FFV1CONTENT_YES)
+        #undef MEDIAINFO_TRACE_FFV1CONTENT_NO //MEDIAINFO_TRACE_FFV1CONTENT_YES has priority
+    #endif
+    #if defined(MEDIAINFO_TRACE_FFV1CONTENT_YES)
+        #define MEDIAINFO_TRACE_FFV1CONTENT 1
+    #else
+        #define MEDIAINFO_TRACE_FFV1CONTENT 0
+    #endif
+#endif
+#if MEDIAINFO_TRACE_FFV1CONTENT && !MEDIAINFO_TRACE
+    #define MEDIAINFO_TRACE_FFV1CONTENT 0 //No trace if MEDIAINFO_TRACE is 0
+#endif
 #if !defined(MEDIAINFO_FILTER)
     #if defined(MEDIAINFO_FILTER_NO) && defined(MEDIAINFO_FILTER_YES)
         #undef MEDIAINFO_FILTER_NO //MEDIAINFO_FILTER_YES has priority
@@ -233,6 +252,26 @@
         #define MEDIAINFO_MD5 1
     #endif
 #endif
+#if !defined(MEDIAINFO_SHA1)
+    #if defined(MEDIAINFO_SHA1_NO) && defined(MEDIAINFO_SHA1_YES)
+        #undef MEDIAINFO_SHA1_NO //MEDIAINFO_SHA1_YES has priority
+    #endif
+    #if defined(MEDIAINFO_SHA1_NO)
+        #define MEDIAINFO_SHA1 0
+    #else
+        #define MEDIAINFO_SHA1 1
+    #endif
+#endif
+#if !defined(MEDIAINFO_SHA2)
+    #if defined(MEDIAINFO_SHA2_NO) && defined(MEDIAINFO_SHA2_YES)
+        #undef MEDIAINFO_SHA2_NO //MEDIAINFO_SHA2_YES has priority
+    #endif
+    #if defined(MEDIAINFO_SHA2_NO)
+        #define MEDIAINFO_SHA2 0
+    #else
+        #define MEDIAINFO_SHA2 1
+    #endif
+#endif
 #if !defined(MEDIAINFO_DEMUX)
     #if !defined(MEDIAINFO_DEMUX_NO) && !defined(MEDIAINFO_DEMUX_YES) && !MEDIAINFO_EVENTS
         #define MEDIAINFO_DEMUX_NO //MEDIAINFO_DEMUX is disabled by default if MEDIAINFO_EVENTS is set to 0
@@ -283,6 +322,20 @@
         #define MEDIAINFO_READTHREAD 1
     #endif
 #endif
+#if !defined(MEDIAINFO_FIXITY)
+    #if defined(MEDIAINFO_FIXITY_NO) && defined(MEDIAINFO_FIXITY_YES)
+        #undef MEDIAINFO_FIXITY_NO //MEDIAINFO_FIXITY_YES has priority
+    #endif
+    #if defined(MEDIAINFO_FIXITY_NO)
+        #define MEDIAINFO_FIXITY 0
+    #else
+        #if MEDIAINFO_ADVANCED
+            #define MEDIAINFO_FIXITY 1
+        #else //MEDIAINFO_ADVANCED
+            #define MEDIAINFO_FIXITY 0
+        #endif //MEDIAINFO_ADVANCED
+    #endif
+#endif
 
 //***************************************************************************
 // Precise configuration
@@ -484,9 +537,15 @@
 #if !defined(MEDIAINFO_MULTI_NO) && !defined(MEDIAINFO_UMF_NO) && !defined(MEDIAINFO_UMF_YES)
     #define MEDIAINFO_UMF_YES
 #endif
+#if !defined(MEDIAINFO_MULTI_NO) && !defined(MEDIAINFO_VBI_NO) && !defined(MEDIAINFO_VBI_YES)
+    #define MEDIAINFO_VBI_YES
+#endif
 #if !defined(MEDIAINFO_MULTI_NO) && !defined(MEDIAINFO_WM_NO) && !defined(MEDIAINFO_WM_YES)
     #define MEDIAINFO_WM_YES
 #endif
+#if !defined(MEDIAINFO_MULTI_NO) && !defined(MEDIAINFO_WTV_NO) && !defined(MEDIAINFO_WTV_YES)
+#define MEDIAINFO_WTV_YES
+#endif
 #if !defined(MEDIAINFO_MULTI_NO) && !defined(MEDIAINFO_XDCAM_NO) && !defined(MEDIAINFO_XDCAM_YES)
     #define MEDIAINFO_XDCAM_YES
 #endif
